import { useSyncExternalStore } from 'use-sync-external-store/shim';

const __DEV__ = process.env.NODE_ENV !== 'production';
const isObj = (val) => {
    return Object.prototype.toString.call(val) === '[object Object]';
};
let isGetStateInMethod = false;
let run = (fn) => {
    fn();
};
const resso = (obj) => {
    if (__DEV__ && !isObj(obj)) {
        throw new Error('object required');
    }
    const state = {};
    const methods = {};
    Object.keys(obj).forEach((key) => {
        const initVal = obj[key];
        if (initVal instanceof Function) {
            methods[key] = (...args) => {
                isGetStateInMethod = true;
                const res = initVal(...args);
                isGetStateInMethod = false;
                return res;
            };
            return;
        }
        const listeners = new Set();
        state[key] = {
            subscribe: (listener) => {
                listeners.add(listener);
                return () => listeners.delete(listener);
            },
            getSnapshot: () => obj[key],
            setSnapshot: (val) => {
                if (val !== obj[key]) {
                    obj[key] = val;
                    run(() => listeners.forEach((listener) => listener()));
                }
            },
            useSnapshot: () => {
                return useSyncExternalStore(state[key].subscribe, state[key].getSnapshot, state[key].getSnapshot);
            },
        };
    });
    const setState = (key, val) => {
        if (key in obj) {
            if (key in state) {
                const newVal = val instanceof Function ? val(obj[key]) : val;
                state[key].setSnapshot(newVal);
            }
            else if (__DEV__) {
                throw new Error(`\`${key}\` is a method, can not update`);
            }
        }
        else if (__DEV__) {
            throw new Error(`\`${key}\` is not initialized in store`);
        }
    };
    return new Proxy((() => undefined), {
        get: (_target, key) => {
            if (key in methods) {
                return methods[key];
            }
            if (key in state) {
                if (isGetStateInMethod) {
                    return obj[key];
                }
                try {
                    return state[key].useSnapshot();
                }
                catch (err) {
                    return obj[key];
                }
            }
            if (__DEV__) {
                if (key !== 'prototype' && key !== 'name' && key !== 'displayName') {
                    throw new Error(`\`${key}\` is not initialized in store`);
                }
            }
        },
        set: (_target, key, val) => {
            setState(key, val);
            return true;
        },
        apply: (_target, _thisArg, [firstArg, oneAction]) => {
            if (typeof firstArg === 'string') {
                setState(firstArg, oneAction);
                return;
            }
            if (isObj(firstArg)) {
                const newObj = firstArg;
                Object.keys(newObj).forEach((key) => {
                    setState(key, newObj[key]);
                });
                return;
            }
            if (typeof firstArg === 'function') {
                const newObj = firstArg(obj);
                Object.keys(newObj).forEach((key) => {
                    setState(key, newObj[key]);
                });
            }
        },
    });
};
resso.config = ({ batch }) => {
    run = batch;
};

export { resso as default };
